# Vue项目实战(移动端)

## 相关资料

1. vue-cli脚手架: [vue2脚手架](https://cli.vuejs.org/zh/)
2. vue3脚手架: [vite](https://vitejs.cn/)
3. vue官网: [https://cn.vuejs.org/v2/guide/index.html
4. vscode插件
   - vetur 必备工具
   - vue-helper 一些辅助功能
   - Vue 3 Snippets 片段

## (零) Es6模块化

### 常用模块化规范:

1. commonjs规范 nodejs遵守commonjs规范(略)

2. AMD规范  reqire.js遵守AMD规范(了解)

   ```
   / 定义模块
   define(function (){  
       var add = function (x,y){  
           return x+y;  
       };  
       return {  
           add: add  
       };  
   }); 
   
   
   // 导入模块
   require(['jquery', 'underscore', 'backbone'], function ($, _, 
   Backbone){  
   　　// some code here  
   });  
   ```

3. CMD规范  sea.js遵守CMD规范(玉伯, 淘宝)

4. ES6规范 

 

### ------以下是Es6的模块规范------

### (1) 导出和导入方式(一)

- export    // 导出

- import {xx} from xxx  // 导出名称是什么, 导入名称也是什么

  ```javascript
  // 导出模块 utils/aa.js
  export let username = '张三';
  export let age = 18;
  ​
  // 导入
  import {username,age} from './utils/aa'
  ```

### (2) 导出和导入方式(一)

- export default // 默认导出

- import xx from xx  // 导入,可以使用任意名称接收默认导出的变量

  #### 例1. export default举例

  ```
  // 导出 utils/bb.js
  export default {
      username: 'zhangsan', 
      age: 18
  }
  ​
  ​
  // 导入
  import obj from './utils/bb'
  ```

  #### 例2. 同时有export和export default

  ```javascript
  // 导出 utils/bb.js
  export let className = 'web0815';
  ​
  export default {
      username: 'zhangsan', 
      age: 18
  }
  ​
  // 导入 app.vue
  import obj,{className} from './utils/bb'
  ```

### (3) 导入时进行重命名

- import {aa as xx} from xxx  // 导入之后重命名

- import * as xx from xxx  // 导入之后重命名

  ```
  // 导入的className重新命名为aaa
  import { className as aaa } from "./utils/bb";
  ​
  // 把utils/aa.js的所有变量导入近来并重命名为obj
  import * as obj from './utils/aa'
  ```

## (一) 创建项目

### 01 安装vue-cli脚手架

注意: node 13.14版本只能安装4.x的版本

```
npm install -g @vue/cli@4
```

### 02 查看vue脚手架版本

出现版本号表示成功

```
vue --version
```

### 03 创建一个新项目

创建项目

```
vue create fresh  // fresh是项目名称
```

使用vscode打开项目文件夹, 运行命令

```
npm run serve        
```

### 04 项目结构

1. 此项目是用vueCli脚手架创建出来的项目
2. 是一个SPA(单页应用), 只有一个html文件
3. src 源码文件夹
   - assets 放静态资源文件
   - components 放公共组件, 比如底部的tab栏
   - App.vue  
     - .vue的文件都是组件, 一个文件就是一个组件, 称为单文件组件
     - App.vue 是根组件
   - main.js   入口文件, 代码运行的入口

 

## (二) 禁用Eslint

```
// 根目录新增vue.config.js
module.exports = {
    lintOnSave: false
}
```

如果vue组件提示红色错误,如下图 ![image.png](http://ddoc.huruqing.cn/assets/1632534908608-d95c94f9-c2e0-4630-b2bc-e2ff1b91bc62.9fd9d798.png)

解决办法: 文件 -> 首选项 -> 设置 然后输入eslint -> 选择Vetur -> 把√取消即可

![img](http://ddoc.huruqing.cn/assets/1632534832966-022d87ff-456d-4cf9-97f2-b74062fd1ede.fed06924.png)

## (三) devtool

vue开发调试工具

(1) 下载插件

- 下载地址:  [http://soft.huruqing.cn](http://soft.huruqing.cn/)
- 极简插件: https://chrome.zzzmh.cn/#/index

(2) 安装插件

1. 解压插件
2. 打开谷歌浏览器 => 竖向省略号 => 更多工具 => 扩展程序
3. 把插件拖进浏览器进行安装

(3) 使用

app.vue里定义变量msg

```
<template>
  <div id="app">
      <p>{{msg}}</p>
      <button @click="msg=22222">修改</button>
  </div>
</template>

<script>  
export default {
  name: 'App',
  data() {
    return {
      msg: 'hello world'
    }
  } 
}
</script> 
```

## (四) 添加less支持

1. 安装模块

   ```
   npm install less less-loader@6.0.0 --save-dev
   ```

2. 在vue文件这样写即可, scoped表示样式只在当前文件有效, 不会影响其他组件

   ps: less-loader要安装6.0版本, 不然有兼容问题

   ```
   <style lang="less" scoped> 
   .box {
     .text {
       color: red;
     }
   } 
   </style>
   ```

## (五) vue路由配置

### 模块划分

### (1)一个简单路由配置

1. `npm i vue-router@3`  安装路由插件
2. 在src创建views文件夹, 创建各个模块的组件
3. 在src内创建router文件夹, 新建index.js(代码如下)
4. 在main.js里, 把router挂载到vue的实例
5. 配置路由出口, 详见下方**第(2)点router-view**
6. 使用router-link进行跳转, 详见下方**第(3)点路由跳转**

 

步骤一: 配置并导出路由模块

```javascript
import Vue from "vue";
import Router from "vue-router";
Vue.use(Router);

// 创建Router的实例, 并导出模块

// 路由数组
const routes = [
  {
    path: "/goods",
    component: () => import("../views/goods/goodsList.vue")
  },
  {
    path: "/type",
    component: () => import("../views/goods/type.vue")
  },
  {
    path: "/cart",
    component: () => import("../views/cart/cart.vue")
  },
  {
    path: "/my",
    component: () => import("../views/my/my.vue")
  }
];

const router = new Router({
  routes
});

export default router;
```



步骤二: 挂载路由

```
// main.js 代码
import Vue from 'vue'
import App from './App.vue'
import router from './router/index'

Vue.config.productionTip = false

new Vue({
  // 把router挂载到vue实例
  router,
  render: h => h(App),
}).$mount('#app')
```



步骤三: 配置子路由出口 (把子路由对应的组件渲染在什么地方)

### (2) router-view

1. 路由出口
2. 路由匹配到的组件将渲染在这里
3. 在app.vue配置

```
<template>
  <div id="app"> 
    <!-- 路由出口 -->
    <router-view></router-view>
  </div>
</template>

<script>
export default {
  name: "App",
  components: {},
};
</script>
```



### (3) 路由跳转()

```
// 方式一
<router-link to="/cart">cart</router-link>
// 方式二
this.$router.push('/cart');
```

实现底部tab栏

```
<template>
  <div id="app">
    <!-- 子路由出口(子路由对应的组件展示在这里) -->
    <router-view></router-view>

    <footer>
      <router-link to="/goods">首页</router-link>
      <router-link to="/type">分类</router-link>
      <router-link to="/cart">购物车</router-link>
      <router-link to="/my">我的</router-link>
    </footer>
  </div>
</template>

<script>
import * as obj from "./utils/aa";

export default {
  name: "App",
  data() {
    return {
      obj
    };
  }
};
</script>

<style lang="less">
body {
  background: #f4f4f4;
}
footer {
  position: fixed;
  bottom: 0;
  height: 50px;
  background: #fff;
  display: flex;
  justify-content: space-around;
  align-items: center;
  width: 100%;
}
</style>
```



 

### (4) 子路由配置

#### 01 子路由说明

使用子路由进行模块路由配置，结构比较分明 比如我们的网站有商品模块,有列表页面和详情页面, 路由如下

/goods商品模块总路由 

/goods/list   商品列表子路由 

/goods/detail   商品详情子路由

```
  {
    path: "/goods",
    redirect: "/goods/list",
    component: () => import("../views/goods/goods.vue"),
    // 配置goods的子路由
    children: [
      {
        path: "list",  // 完整路径 /goods/list
        component: () => import("../views/goods/children/goodsList.vue"),
      },
      {
        path: "detail",  // 完整路径 /goods/detail
        component: () => import("../views/goods/children/goodsDetail.vue"),
      },
    ]
  }, 
```



#### 02 在需要在模块根组件配置子路由出口

goods.vue

```
<template>
  <router-view></router-view>
</template> 
```

### (5) active-class

active-class是vue-router模块的router-link组件中的属性，用来做选中样式的切换；

1. 只要路由中包含to里面的路由, 就能匹配到, 就会高亮, 比如: /product, /product/list, /product/detail都会使下面的第二个router-link高亮
2. exact 表示精确匹配, 只有路由完全一样才能被匹配

```
<footer>
    <router-link active-class="active" to="/goods">首页</router-link>
    <router-link active-class="active" to="/type">分类</router-link>
    <router-link active-class="active" to="/cart">购物车</router-link>
    <router-link active-class="active" to="/my">我的</router-link>
</footer>
```

### (6) history模式

**vue2配置方式**

1. vue-router 默认 hash 模式 —— 使用 URL 的 hash 来模拟一个完整的 URL，于是当 URL 改变时，页面不会重新加载。

2. 如果不想要很丑的 hash，我们可以用路由的 **history 模式**，这种模式充分利用 history.pushState API 来完成 URL 跳转而无须重新加载页面

3. 使用history需要后端支持, vue-cli创建的devServer可以支持

   ```
   const router = new VueRouter({
     mode: 'history',  // 默认hash
     routes: [...]
   })
   ```

### (7) redirect重定向

当访问 '/', 我们使用redirect使它默认跳到 '/product'

```
{
    path: '/',
    redirect: '/product'
},
```

### (8) 404配置

假如用户访问了一个没有的路由, 我们让它跳转到404页面

```
  {
    path: '*',
    component:()=>import('@/components/NotFound')
  }
```



## (六)(1) 配置静态资源

### (1) 配置公共样式和字体图标

详看代码

1. 安装插件

   ```
   npm i vant@latest-v2 -S
   ```

2. 使用插件

   ```
   // main.js
   import Vue from 'vue'
   import App from './App.vue'   
   import router from './router/index' 
   import Vant from 'vant';
   // 导入公共样式
   import './assets/style/index.css';
   // 导入vant样式文件
   import 'vant/lib/index.css';
   Vue.use(Vant);
   
   Vue.config.productionTip = false
   
   new Vue({
     router,
     render: h => h(App),
   }).$mount('#app')
   ```

    

## (六)(2) 父子组件通信(背诵)

### 知识点

1. 父传子: 父组件通过(绑定)属性的方式传数据给子组件, 子组件使用props接收数据
2. 子传父: 父组件在子组件上绑定一个自定义事件, 子组件通过$emit触发该自定义事件, 同时可以传入数据

### (1) 组件的概念

1. 可以将页面的某一部分做成一个组件, 比如页面的导航栏, 底部的tab栏等, 组件可以重复使用
2. 组件可以分为全局组件和局部组件

#### 例1. 定义一个全局组件(理解)

1. 通过`Vue.component()`来定义全局组件

2. 定义全局组件必须放在 `new Vue()`

   ```
   Vue.component('Hello',{
     data() {
       return {
         msg: 'hello world',
       }
     },
     template: `
       <h3>{{msg}}</h3>
     ` 
   });
   ```

3. 需要在vue.config.js进行配置才能生效

   ```
   const path = require('path');
   module.exports = {
     lintOnSave: false,
     configureWebpack: (config) => {
       config.resolve = {
         extensions: [".js", ".vue", ".json", ".css"],
         alias: {
           vue$: "vue/dist/vue.esm.js",
           "@": path.resolve("src")
         }
       };
     }
   };
   ```

#### 例2. 定义一个局部组件

// components/MyButton.vue

```
<template>
  <div class="btn">
     {{msg}}
  </div>
</template>

<script>
export default {
    data() {
        return {
            msg: '我是老胡'
        }
    }
}
</script>

<style scoped>
.btn {
    display: inline-block;  
    padding: 10px 20px;
    background: gray;
    color: #fff;
    text-align: center;
}
</style> 
```

//  

```
<template>
  <div>
    <!-- 使用组件 -->
    <Mybutton />
  </div>
</template>

<script>
// 1. 导入组件
import Mybutton from "../../components/MyButton.vue";
export default {
  // 2. 注册组件
  components: {
    Mybutton
  }
};
</script> 
```

### (2) 父传子

1. 父组件通过属性传数据给子组件

2. 子组件通过props接收父组件的传过来的数据

   注意: 

- 通过普通属性传递的数据类型是字符串
- 通过绑定属性传递的数据类型是所传数据的类型

#### 例1. 父传子举例

// demo.vue

```
<template>
  <div>
     <h1>demo父组件</h1>
     <hr>
     <Son msg="hello world"  :money="money"  :count="true"/>
  </div>
</template>

<script>
import Son from './Son.vue';
export default {
  components: {
    Son
  },

  data() {
    return {
      money: 2000
    }
  }
};
</script>
```

// Son.vue

```
<template>
  <div>
      <h4>这是子组件</h4>
      <!-- 使用 -->
      <p>{{msg}}</p>
      <p>{{money}}</p>
      <p>{{typeof count}}</p>
  </div>
</template>

<script>
export default {
    // 接收msg属性
    props: ['msg','money','count']
}
</script>
```



 

#### 例2. 父传子实践

把生鲜首页拆分成多个子组件

```
<template>
  <div> 
    <Swiper/>
    <Miaosha/>
    <TypeList/> 

    <Footer/>
  </div>
</template>

<script>
import Footer from '@/components/Footer.vue';
import Swiper from "./goodList/Swiper.vue";
import Miaosha from "./goodList/Miaosha.vue";
import TypeList from "./goodList/TypeList.vue";
export default {
  components: {
    Footer,
    Swiper,
    Miaosha,
    TypeList
  }
};
</script> 

```



 

### (3) 子传父

1. 父组件在子组件上绑定一个自定义事件(事件名称我们自己定义的, vue本身是没有这个事件的)
   - 自定义事件再绑定一个函数
   - 这个函数也可以自定义
2. 子组件使用$emit触发(调用)该事件, 并把数据以参数形式传给父组件

#### 例1. 一个简单的例子

// demo.vue

```
<template>
  <div>
    <h3>父组件</h3>
    <hr />
    <Son @aaa="say"/>
  </div>
</template>

<script>
import Son from "./Son";
export default {
  components: {
    Son,
  },
  data() {
    return { 
    };
  },
  methods: {
    say(data) {
      alert(data)
    }
  } 
};
</script> 
```

// Son.vue

```
<template>
  <div>
    <h4>子组件</h4>
    <button @click="handleClick">点击</button>
  </div>
</template>

<script>
export default {
  methods: {
    handleClick() {
      this.$emit("aaa", "我是子组件");
    }
  }
};
</script>
```



 

#### 例2. 自定义导航栏组件

// NavBar.vue

```
<template>
  <div class="nav flex jc-sb aic pl-15 pr-15">
    <!-- 触发左边的事件 -->
    <p @click="$emit('click-left')">
      <van-icon v-if="typeof leftArrow === 'string'" name="arrow-left" />
      <span>{{ leftText }}</span>
    </p>
    <span>{{ title }}</span>
    <!-- 触发右边的事件 -->
    <span @click="handleClick">{{ rightText }}</span>
  </div>
</template>

<script>
export default {
  props: ["left-text", "title", "right-text", "left-arrow"], 

  methods: {
    handleClick() {
      // 做点事情
      console.log(11111111111111); 
      // 触发事件的同时给父组件传了个对象
      this.$emit("click-right", {username: '张三', age:18});
    }
  }
};
</script>

<style lang="less" scoped>
.nav {
  height: 50px;
  border: 1px solid;
}
</style>
```

// demo.vue

```
<template>
  <div>
    <NavBar @click-left="onClickLeft"  @click-right="onClickRight" left-text="返回" title="购物车" right-text="编辑" left-arrow/>
  </div>
</template>

<script>
import NavBar from "@/components/NavBar.vue";
export default {
  components: {
    NavBar
  },

  data() {
    return {
      money: 2000
    };
  },

  methods: {
    onClickLeft() {
      alert("点击左边");
    },
    onClickRight(data) {
      alert(data.username);
    }
  }
};
</script>
```



## (七) 配置axios拦截器(背诵)

1. 对ajax请求进行拦截
   - 在请求头添加token
   - 添加loading等
2. 对ajax响应数据进行拦截
   - 统一处理请求失败的情况, 这样就不需要在每个组件里处理失败的情况
   - 有些接口需要登录才能访问, 在没登录的情况下跳转到登录页面

```
import axios from "axios";
import Vue from "vue";
import { Toast } from "vant";
Vue.use(Toast);

const service = axios.create({
  baseURL: "http://huruqing.cn:3003",
  timeout: 50000, // 请求超时时间(因为需要调试后台,所以设置得比较大)
});

// request 对请求进行拦截
service.interceptors.request.use(
  (config) => {
    // 开启loading
    Toast.loading({
      message: "加载中...",
      forbidClick: true,
      loadingType: "spinner",
    });
    // 请求头添加token
    config.headers["token"] =
      "gg12j3h4ghj2g134kj1g234gh12jh34k12h34g12kjh34kh1g";
    return config;
  },
  (error) => {
    Promise.reject(error);
  }
);

// response 响应拦截器
service.interceptors.response.use(
  (response) => {
    Toast.clear();
    const res = response.data;
    if (res.code == 666) {
      return res;
    } else {
      // 成功连接到后台, 但是没有返回正确的数据
      Toast.fail(res.msg);
    }
  },
  (error) => {
    Toast.clear();
    // 跟后台连接失败
    Toast.fail("网络异常,请稍后再试");
  }
);

export default service;

```

## (八) Sticky 粘性布局

将内容包裹在 `Sticky` 组件内即可。

```
<van-sticky>
  <van-button type="primary">基础用法</van-button>
</van-sticky>
```



## (九) 图片懒加载

 vant提供了自定义指令 v-lazy实现图片懒加载

```
<img v-for="img in imageList" v-lazy="img" />
```



## (十) 全局注册组件

// 注册全局组件除了多了个template之外,其它跟平时写组件类似 

// 注册全局组件要放在new Vue() 之前

```
Vue.component('button-counter', {
  data: function () {
    return {
      count: 0
    }
  },
  template: '<button v-on:click="count++">你打了我 {{ count }} 次</button>'
})

// 在其他组件就可以使用
<template>
    <div>
    <button-counter></button-counter>
  </div>  
</template>
```

```
// 改造checkbox, 官网例子
Vue.component('base-checkbox', {
  model: {
    prop: 'checked',
    event: 'change'
  },
  props: {
    checked: Boolean
  },
  template: `
    <input
      type="checkbox"
      v-bind:checked="checked"
      v-on:change="$emit('change', $event.target.checked)"
    >
  `
})

// 然后就可以像下面这样来使用
<template>
  <div> 
   <base-checkbox v-model="flag"></base-checkbox>
        <p>{{flag}}</p>
  </div>
</template>

<script>
export default {
  data: function () {
    return {
      flag: false
    };
  },
}
</script>
```

```
// 另外需要在根目录的vue.config.js中开启运行时编译
module.exports = {
    runtimeCompiler: true
}
```

## (十一) vue slot插槽

 元素作为承载分发内容的出口, vue的插槽分为三种:

1. 普通插槽 (没有名字)
2. 具名插槽 (有名字)
3. 作用域插槽 (让父组件可以访问插槽的内容)

### vant有赞ui库中slot的例子

```
<van-nav-bar title="标题" left-text="返回" left-arrow> 
   <p slot="right">
     <van-icon name="search" size="18" />
   </p>
</van-nav-bar>
```



### 1. 普通插槽

// 父组件demo.vue代码

```
<template>
  <div>
      <Son>
          <button>卷死别人</button>
      </Son>
  </div>
</template>

<script>
import Son from './Son.vue';
export default {
    components: {
        Son
    }
};
</script>

<script>
import Son from "./Son";
export default {
  components: {
    Son,
  }
};
</script> 
```

// 子组件Son.vue

```
<template>
  <div>
    <slot></slot>
  </div>
</template> 
```



### 2. 具名插槽

#### 例1. 具名插槽举例

// 父组件 demo.vue

```
<template>
  <div>
      <Son>
          <button>卷死别人</button>
          <!-- 以下内容传入到name="aaa"的插槽里 -->
          <button slot="aaa">啊啊啊啊啊啊</button>
      </Son>
  </div>
</template>

<script>
import Son from './Son.vue';
export default {
    components: {
        Son
    }
};
</script>
```

// 子组件 Son.vue

```
<template>
    <div>
        <h4>子组件</h4>
        <!-- 普通插槽, 接收来自父组件的内容 -->
        <slot></slot>
        <!-- 具名插槽 -->
        <slot name="aaa"></slot>
    </div>
</template> 
```

#### 例2. vant的具名插槽

```
<template>
  <div>
    <van-nav-bar title="首页">
      <div slot="left">
        <button>返回</button>
      </div> 

      <template #right>
        <van-icon name="search" size="18" />
      </template>
    </van-nav-bar>
  </div>
</template>
```



### **3. 作用域插槽**

- 通过作用域插槽可以让父组件访问子组件插槽的数据(是子传父的一种方式)
- 使用slot-scope="scope" 或 v-slot="scope"来获取插槽的作用域

// 父组件 demo.vue

```
<template>
  <div>
      <!-- 作用域插槽 -->
      <Son>
          <!-- 写法1 -->
          <!-- <template v-slot="scope">  -->
          <!-- 写法2 -->
          <template slot-scope="scope"> 
            <li>{{scope.user.enname}}</li>
            <li>{{scope.user.cnname}}</li> 
          </template>
      </Son>
  </div>
</template>

<script>
import Son from './Son.vue';
export default {
    components: {
        Son
    }
};
</script>
```

// 子组件 Son.vue

```
<template>
  <div>
    <slot :user="user">子组件数据：{{ user.enname }}</slot>
  </div>
</template>

<script>
export default {
  data() {
    return {
      msg: "hello",
      user: {
        enname: "zhangsan",
        cnname: "张三"
      }
    };
  }
};
</script>

```



## (十二)(1) 使用ui库需要关注的三点

以vant 的导航栏组件van-nav-bar为例

1. 属性, 该组件提供了哪些绑定属性
2. 事件, 该组件提供了哪些事件
3. 插槽, 该组件提供了哪些插槽

## (十二)(2) 封装组件注意事项

### (1) 组件封装分类

1. 给自己用, 此组件只是为某个页面服务, 比如首页封装商品列表的组件

   给自己用, 子组件可以有相对复杂的逻辑

2. 给大家一起用, 此组件是整个项目都可以使用, 比如导航栏

   要多考虑组件的通用性

### (2) 考虑三个方面问题

1. 使用组件时传入什么属性(绑定属性)
2. 组件提供什么事件
3. 若允许调用这自定义内容, 需要使用插槽

### (3) 组件间的通信技巧

1. 需要共享的数据定义父组件
2. 数据的修改等操作在父组件里完成(主要的逻辑代码都放在父组件)
3. 子组件, 主要用来展示, 提供事件 

## (十三) 三种路由传参方式(背诵)

**知识点:**

1. 通过query传参, $route.query接收参数
2. 通过params传参, 使用$route.params接收参数
3. 动态路由传参, 使用$route.params接收参数

​    注意: router和route不是一回事

 

### 例1. path+query传参

- 跳转时把参数拼接在url的后面
- 通过$route.query来获取参数

// demo.vue

```
<template>
 <div>
   <router-link to="/demo2?aa=1000">router-link方式</router-link>
   <br/>
    <button @click="goto1">不传参</button>
    <button @click="$router.push('/demo2')">不传参2</button>
    <br/>
    <button @click="goto2">传参</button>

 </div>
</template>


<script>
  export default { 
    methods: {
      goto1() {
        this.$router.push('/demo2');
      },
      goto2() {
        this.$router.push({
          path: '/demo2',
          query: {
            username: '张三',
            age: 18
          }
        });
      }
    },
  }

</script> 
```

// demo2.vue

```
<template>
 <div>
   <h3>这是demo2</h3>
   <p>
    {{$route.query}} 
    </p>
 </div>
</template> 

<script>
  export default {
    created() {
      console.log(this.$route.query);
    }
  }

</script> 
```



 

### 例2. 通过name+params传参

- 配置路由的时候, 要给每一个路由添加name属性
- 给路由添加name的时候, name要唯一,  一般name的命名采用 模块名+路由名, 比如 "goods-list"

跟query传参的区别:

- 参数不会拼接在url上
- 刷新之后,query参数还存在, params参数不存在

```
// demo2的路由要添加name
{
    path:'/demo2',
    name: 'demo2',
    component:()=> import('../views/demo2/demo2.vue')
},
```

// demo.vue

```
<template>
 <div>
    <button @click="goto">params传参</button>
 </div>
</template>


<script>
  export default { 
    methods: { 
      goto() {
        this.$router.push({
          name: 'demo2',
          params: {
            username: '张三',
            age: 18
          }
        });
      }
    },
  }

</script> 
```

// demo2

```
<template>
 <div>
   <h3>这是demo2</h3>
   <p>
    {{$route.params}} 
    </p>
 </div>
</template> 

<script>
  export default {
    created() {
      console.log(this.$route.params);
    }
  }

</script> 
```



### 例3. 动态路由传参

- 跳转时, 参数拼接在url上, 不过不是用?号, 是用 "/xxx"
- 接收参数使用$route.params进行接收

```
{
    // 在path上配置参数的名称username
    path:'/demo2/:username',
    name: 'demo2',
    component:()=> import('../views/demo2/demo2.vue')
},
```

// demo.vue

```
<template>
 <div>
     <router-link to="/demo2/张三">动态路由传参</router-link>
 </div>
</template> 
```

// demo2.vue

```
<template>
 <div>
   <h3>这是demo2</h3>
   <p>
    {{$route.params}} 
    </p>
 </div>
</template>  
```

## (十四) vuex(背诵)

### (1) vuex是什么

Vuex 是一个专为 Vue.js 应用程序开发的**状态管理模式**。它采用集中式存储管理应用的所有组件的状态，并以相应的规则保证状态以一种可预测的方式发生变化。

总结: vuex是用来共享组件状态的, 是跨组件通信最常用的一种方式, 具体的做法就是使用Vuex创建一个仓库, 在仓库里定义我们要共享的状态, 用vuex提供的方法去获取和修改仓库里的状态

### (2) 核心概念

1. state 定义状态
2. getters 派生状态
3. mutation 修改状态(同步)
4. action  修改状态(异步)
5. module 模块化

 

### (3) vuex应用

#### 例1. 基础应用(state, mutation)

- 创建仓库
  1. 安装vuex `npm i vuex --save`
  2. 创建仓库
  3. 挂载仓库

```
// 1. src/store/index.js
import Vue from 'vue';
import Vuex from 'vuex';
Vue.use(Vuex);

// 创建仓库
const store = new Vuex.Store({

});

export default store;


// 2. 挂载到根实例 /src/main.js
import router from "./router/index";
import store from './store/index';

Vue.use(Vant);  

Vue.config.productionTip = false;
new Vue({
  store,
  router,
  render: (h) => h(App),
}).$mount("#app");
```

- **定义状态**

```
const store = new Vuex.Store({
  // 定义状态
  state: {
    username: "张三",
    age: 100,
  },
});
```

- **获取状态**
  1. 直接获取 `this.$store.state.username`

```
<template>
  <div>
    <p>username: {{$store.state.username}}</p>
  </div>
</template>   
<script> 
export default {
    created() {
      console.log(this.$store.state);
    } 
};
</script>
```

​		 2. 通过mapState获取, mapState是vuex提供的方法, 可以让我们更方便的获取属性

```
<template>
  <div>
    <p>username: {{username}}</p>
    <p>age: {{age}}</p>
  </div>
</template>  

<script>
import {mapState} from 'vuex';
export default { 
    computed: {
      ...mapState(['username','age'])
    }
};
</script>
```

- **修改状态: 通过mutation进行修改**
  - 修改状态只能通过mutation来修改, 不可以直接修改
  - mutation只支持同步操作
- 步骤:
  - 定义mutation
  - 提交mutation

```
// 1.定义mutation
import Vue from "vue";
import Vuex from "vuex";
Vue.use(Vuex);
// 创建仓库
const store = new Vuex.Store({
  // 定义状态
  state: {
    username: "张三",
    age: 100,
  },
  // 定义mutaion
  mutations: {
      /**
       * 定义修改username的muation 
       * @param {*} state 状态
       * @param {*} payload 传入的新数据
       */
      set_username(state,payload) {
        state.username = payload;
      },

      // 定义修改age的mutation
      set_age(state,payload) {
        state.age = payload;
      }
  } 
});

export default store;


// 2. 提交mutaion
<template>
  <div>
    <p>username: {{$store.state.username}}</p>
    <button @click="change">修改状态</button>
  </div>
</template>   
<script> 
export default {  
    methods: {
      change() {
        // 提交mutation,参数1 mutation的名称, 参数2 新的数据
        this.$store.commit('set_username','李四'); 
      }
    }
};
</script>

```

#### 例2. 进阶应用-getters

类似计算属性computed, 用来**派生**状态

```
// 1.  在 src/store/index.js
import Vue from 'vue';
import Vuex from 'vuex';
Vue.use(Vuex);

// 创建仓库
const store = new Vuex.Store({
    // 定义状态
    state: {  
        IDcard: '410221199009025977',
    },

    getters: {
        birthdate(state) {
            let date = state.IDcard.slice(6,14);
            let str = date.replace(/^(\d{4})(\d{2})(\d{2})$/,'$1-$2-$3')
            return str;
          }
    }, 
})

export default store;
    
// 2. 在组件里使用
    
<template>
    <div> 
      {{birthdate}}
  </div>  
  
</template>    
    
import {mapGetters} from 'vuex';
export default {
 computed:{
    ...mapGetters(['birthdate'])
 } 
}
```



#### 例3. 进阶应用-action异步修改状态

1. 定义状态
2. 定义mutation, 通过mutation来修改状态
3. 定义action , 通过action来提交(commit)mutation
4. 用户派发action

```
import Vue from 'vue';
import Vuex from 'vuex'; 
Vue.use(Vuex);
import $axios from '@/api/index';

// 创建仓库
const store = new Vuex.Store({ 
    // 定义状态
    state: { 
        productList: [], 
    },  
    mutations: { 
        set_productList(state, payload) {
            state.productList = payload;
        }
    },

    actions: {
        PRODUCTLIST(store,currPage) {
            // action可以做异步操作, 可以发请求
            $axios.get('/product/list?currPage='+currPage).then(res=> {
                store.commit('set_productList', res.list);
            }) 
        }
    }
})

export default store;
```

// 使用

```
<template>
  <ul>
    <button @click="handleClick(1)">获取列表</button>
    <button @click="handleClick(2)">获取列表</button>
    <button @click="handleClick(3)">获取列表</button>
    <li v-for="item in productList" :key="item.productId">{{item.masterName}}</li>
  </ul>
</template>

<script>
import $axios from "@/api/index";
import {mapState} from 'vuex';
export default {  

  computed: {
    ...mapState(['productList'])
  },
  
  methods: {
    handleClick(page) {
      this.$store.dispatch('PRODUCTLIST',page);
    }
  }
};
</script>
```



#### 例4. 进阶应用-模块化

```
// 1.定义模块的state getters mutaions actions
// src/store/modules/cart.js
export default {
    state: {
       cartNum: 100 
    },

    getters: { },
    mutaions: { },
    actions: {}
}
// src/store/modules/type.js
export default {
    state: {
       aaa: 333 
    },

    getters: { },
    mutaions: { },
    actions: {}
}


// 2.合并模块
import cart from './modules/cart';

const store = new Vuex.Store({
  modules:{
    cart,
    type, 
  },
}
                             
// 3.使用(在任何一个组件内)
<template>
    <div>
      {{ $store.state.cart.cartNum }}
      {{$store.state.type.aaa}}                            
  </div>                             
</template>                             
```



#### 例5. 进阶应用-vuex持久化

持久化原理是将state存入本地缓存,  刷新之后, 直接先从缓存读取

1. 安装插件 `npm i vuex-persistedstate -S`
2. 应用插件: 可以设置使用哪一种缓存, 可以设置只缓存哪些状态

```
import Vue from "vue";
import Vuex from "vuex";
// 导入持久化插件
import createPersistedState from "vuex-persistedstate"; 
Vue.use(Vuex); 

const store = new Vuex.Store({
  plugins: [createPersistedState()],
})
```



## (十五) 微信支付-轮询和websocket

### (1) 微信支付流程

1. 用户点击提交订单, 商户(服务器端)创建订单, 并返回订单信息和支付二维码给用户
2. 用户扫码支付(货值调起微信支付)
3. 支付平台收到钱后, 返回支付信息给用户, 同时通知商户(服务器端)已收到用户的钱
4. 商户(服务器端)修改订单的状态
5. 用户(web端)获取支付结果, 得到结果后做相应操作
   1. 轮询方式
   2. websocket

 

### (2) 获取支付结果的两种方式

获取支付结果, 可以使用轮询或者websocket

1. 轮询, 定时给服务器请求, 询问结果, 直到有结果为止, 轮询不需要服务器特别的支持
2. websocket, 前端只需跟后台建立连接即可(长连接), 有了结果服务器可以给前端主动推送信息, websocket是长连接, 而http请求是一次性连接, websocket需要服务器端创建socket接口, 很多网站的客服服务就是使用websocket做的

```
// 轮询
<template>
  <div class="payment pay"></div>
</template>

<script>
export default {
  data() {
    return {
      timer: null,
      orderId: 'sdfasdfasdfasdfasdfasdfas'
    };
  },

  created() {
    this.waitResult();
  },

  beforeDestroy() {
    // 销毁定时器
    clearInterval(this.timer);
  },

  methods: {
    async waitResult() {
      // 创建定时器
      this.timer = setInterval(async () => {
        let res = await this.$axios.post("/order/detail", {
          orderId: this.orderId,
        });
        if (res.result.orderStatus === "01") {
          clearInterval(this.timer);
          // 支付成功, 返回首页
          this.$router.push("/");
        }
      }, 2000);
    },
  },
};
</script> 
```

```
// webSocket
<template>
  <div>{{result}}</div>
</template>

// webSocket
<template>
  <div>{{result}}</div>
</template>

<script>
export default { 
  data() {
    return {
      result: ''
    }
  },

  created() {
    this.connect();
  },
  methods: {
    connect() {
      this.result = '等待支付结果...';
      // 跟后端建立连接
      var ws = new WebSocket("ws://huruqing.cn:3003/socket");
      // onopen连接结果
      ws.onopen = () => {
        console.log("连接成功");
      };
      // 等待后端推送信息
      ws.onmessage = (res) => {  
        this.result = res.data;
      };
    },
  },
};
</script> 
```

## (十六) 进入组件, 滚动条回顶部

解决办法

```
// router/index.js
const routes = [...];
const router = new Router({
  mode: "history",
  scrollBehavior: () => ({
    y: 0
  }),
  routes
});
```



## (十七) ref获取dom节点和子组件实例

1. ref可以获取原生dom节点和子组件实例

```
<template>
  <div>
    <span ref="demo">ref例子</span> 
    <button @click="handleClick">点击</button>
    <hr />
    <Son ref="son" />
  </div>
</template>

<script>
import Son from "./Son.vue";
export default {
  components: {
    Son,
  },
  methods: {
    handleClick() {
      console.log(this.$refs.demo.innerText);
      console.log(this.$refs.son.msg);
    },
  },
};
</script> 

// Son.vue
<template>
    <div>
        {{msg}}
    </div> 
</template>

<script>
export default { 
    data() {
        return {
            msg: 'hello',
            title: '2222222'
        }
    }
}
</script>
```

​	2.应用

```
// 父组件控制子组件的显示和隐藏(子组件无状态)
// demo.vue
<template>
  <div>
    <button @click="show = !show">点击</button>
    <hr>
    <Son :show="show" />
  </div>
</template>

<script>
import Son from "./Son.vue";
export default {
  components: {
    Son,
  },
  data() {
    return {
      show: true,
    };
  },
};
</script> 

// Son.vue
<template>
  <div v-if="show">
      <p>子组件内容</p>
      <p>子组件内容</p>
      <p>子组件内容</p>
      <p>子组件内容</p>
  </div>
</template>

<script>
export default {
    props: ['show'] 
}
</script> 
```

```
// 父组件控制子组件的显示和隐藏(子组件有状态)
<template>
  <div> 
    <button @click="$refs.aaa.show=!$refs.aaa.show">显示弹窗</button>  
    <Alert ref="aaa"/>
  </div>
</template>

<script> 
import Alert from './Alert.vue'

export default {  
  components: {
    Alert
  }, 
};
</script>


// Son.vue(假设子组件是别人设计的组件, 多处地方在使用, 所以并不适宜去改动, 不然容易改出bug, 子组件有个show的状态来控制其显示和隐藏
<template>
  <div v-if="show">
      <p>{{msg}}</p>
      <p>{{msg}}</p>
      <p>{{msg}}</p>
  </div>
</template>

<script>
export default { 
  data() {
    return {
      show: true,
      msg: 'hello vue'
    }
  }
}
</script> 

```

\## 

## ----------进阶知识---------

## (一) 模拟数据接口

### 为什么要模拟数据接口

1. 后端接口没开发好, 只能先使用mock数据
2. 有些时候调试代码时使用模拟接口会更方便

### (1) 常见模拟数据的方式

1. 使用json数据(略)
2. 使用nodejs自己搭建服务器(略)
3. 使用json-server创建服务器
4. 使用mockjs拦截请求,实现数据的模拟
5. 线上模拟数据接口(略)

 

### (2) 详细配置方法

#### 01 json-server的配置

1. 文档地址:  https://www.npmjs.com/package/json-server
2. npm i json-server -g    //全局安装
3. 根目录创建db.json
4. 启动json-server

```
json-server --watch db.json
```

```
// db.json
{
  "posts": [
    { "id": 1, "title": "json-server", "author": "typicode" }
  ],
  "comments": [
    { "id": 1, "body": "some comment", "postId": 1 }
  ],
  "profile": { "name": "typicode" }
}
```

​	5.访问接口

```
http://localhost:3000/posts/1
```

​	6.将命令添加到package.json, 可以使用 npm run json 启动项目

```
 "scripts": {
    "json": "json-server --watch db.json" 
  },
```

#### 02 mockjs拦截模拟接口

安装mockjs `npm i mockjs`

第一步: 新建文件 /api/mock.js

```
import Mock from 'mockjs';
// 判断环境, 模拟数据只在开发环境中进行
let isDev = process.env.NODE_ENV === 'development';
if (isDev) {
    const files = require.context('../views', true, /mock\.js$/);
    let list = files.keys();
    list.forEach(key => {
        let { path, data, close } = files(key).default;
        debugger;
        if (!close) {
            // get请求
            Mock.mock(RegExp(path + ".*"), 'get', config => {
                return data;
            });
            // post请求
            Mock.mock(RegExp(path + ".*"), 'post', config => {
                return data;
            });
        }
    })
}

```

第二步, 在main.js中导入文件

```
import './api/mock';
```

第三步, 在各个模块内先建mock.js文件, 比如cart模块

- path, 接口地址
- data, 返回的数据

```
export default {
    path: "/cart/all",
    close: false, // 是否关闭拦截
    data: {
        "code": 666,
        "msg": "success", 
        "list": [{
            "productId": "105987280",
            "buyNum": 6,
            "masterName": "【山东】金秋红蜜桃",
            "slaveName": "基地果园现摘现发 鲜甜红润肉厚 富含维生素",
            "userId": "60d6f0b08f119d3e25af8cef",
            "imgUrl": "https://mall.s.maizuo.com/86d8414272e50b4b9a7b185ac30dfc86.png",
            "price": "5780",
            "cartId": "634e5d85eae4ee46d238e1a7"
        }, {
            "productId": "104015233",
            "buyNum": 1,
            "masterName": "【海南】红心火龙果礼盒装  5斤装（5-7枚）",
            "slaveName": "基地果园采摘 鲜嫩爽口水润 富含天然花青素",
            "userId": "60d6f0b08f119d3e25af8cef",
            "imgUrl": "https://mall.s.maizuo.com/14a9e655a578abe935f8c150b6d85592.png",
            "price": "6990",
            "cartId": "634e5d8aeae4ee46d238e1a8"
        }, {
            "productId": "102062396",
            "buyNum": 1,
            "masterName": "【山东】海阳网纹瓜 2个5斤装（2.4-2.6斤/枚）",
            "slaveName": "皮薄圆润均匀 肉厚鲜嫩脆甜 现摘现发",
            "userId": "60d6f0b08f119d3e25af8cef",
            "imgUrl": "https://mall.s.maizuo.com/bcc961c6faea5b9a2467c9d759db3cd6.png",
            "price": "3580",
            "cartId": "634e97b4eae4ee46d238e2d4"
        }, {
            "productId": "104422476",
            "buyNum": 1,
            "masterName": "【山东】水果牛奶白玉黄瓜 5斤装（18-25根）",
            "slaveName": "农家现摘现发 饱满脆嫩光滑 肉厚新鲜爽口",
            "userId": "60d6f0b08f119d3e25af8cef",
            "imgUrl": "https://mall.s.maizuo.com/fac0e634a3fc27e1708ad06f667aa47d.png",
            "price": "3990",
            "cartId": "634e97b8eae4ee46d238e2d5"
        }, {
            "productId": "101472086",
            "buyNum": 1,
            "masterName": "【湖南】洞口雪峰蜜桔 5斤装（55-70mm果径/枚）",
            "slaveName": "源头新鲜采摘 饱满圆润细腻 酸甜脆嫩爆汁",
            "userId": "60d6f0b08f119d3e25af8cef",
            "imgUrl": "https://mall.s.maizuo.com/2b6281c70abc9b6237dcce883357f6f1.png",
            "price": "3580",
            "cartId": "634e97bbeae4ee46d238e2d6"
        }]
    }
}
```

总结: 使用json-server, nodejs, json的方式都可以查看网络请求, 唯独使用mockjs是查看不到网络请求的, 但使用mockjs的方式更为方便

 

## (二) 计算属性computed和method的区别

computed的作用是可以根据一个属性(或多个)计算出另外一个属性, 跟method的区别是:

1. computed可以缓存计算结果, 依赖的属性没有发生改变, computed里的方法就不会执行
2. methods里的方法, 只要调用就执行

下面的例子中, computed里的totalMoney方法只执行了一次, 而methods里的sum方法执行了三次

```
<template>
  <div>
    <p>{{ totalMoney }}</p>
    <p>{{ totalMoney }}</p>
    <p>{{ totalMoney }}</p> 
    <hr />
    <p>{{ sum() }}</p>
    <p>{{ sum() }}</p>
    <p>{{ sum() }}</p> 
  </div>
</template>

<script>
export default {
  data() {
    return {
      price: 28,
      num: 10,
    };
  },

  computed: {
    totalMoney() {
      console.log("computed");
      return this.price * this.num;
    },
  },

  methods: {
    sum() {
      console.log("methods");
      return this.price * this.num;
    },
  },
};
</script>
```



## (三) watch监听对象

1. 普通的监听方法, 对对象无效
2. watch的参数
   1. handler 监听器(有改变就执行)
   2. immediate 马上执行
   3. deep 深度监听

```
<template>
  <div>
    <input type="text" v-model="obj.username" />
    <p>{{ obj.username }}</p>
  </div>
</template>

<script>
export default {
  data: function () {
    return {
      obj: {
        username: "张三",
      },
    };
  },

  watch: { 
    obj: {
      // 发生改变时执行的函数
      handler(newObj) {
        console.log(newObj.username);
      },
       // 首次绑定watch就执行
      immediate: true,
      // 深层监控,不设置,引用数据类型监控不到
      deep: true
    }
  },
};
</script>
```



## (四) token(令牌)和session(会话)

相同点: 两者都是用来识别用户的

1. session会话, sessionId
   1. 对于特定接口, 前端需要登录才能访问, 所以第一次访问时需要登录, 登录成功, 服务器会返回一个sessionId
   2. 下次前端再访问同一个接口的时候, 把sessionId带上(cookie), 这样服务器就能识别是谁在访问, 如果这个人已经登录过, 就不再需要再登录, session一般设有效期
2. token令牌, 或叫同行证
   1. 前端在登录成功时, 服务器会把用户的相关信息加密, 得到一个密文, 这就是token, 返回给前端
   2. 前端再次访问接口时, 把token带上, 服务器端收到token就对它进行解密, 得到用户信息

 

## (五) vue过滤器

作用: 格式化数据

有全局和组件内的过滤器, 优先使用组件内的过滤

```
// 组件内的过滤器
<template>
  <div>
      {{num | f}}
  </div>
</template>   

<script>
export default {
  data() {
    return {
      num: 10
    }
  }, 
  filters: {
    f(num) {
      return Number(num).toFixed(2);
    }
  }
}
</script>

// 全局过滤器
Vue.filter('fMoney', (money)=> {
  let num = money/100;
  return num.toFixed(2);
})
new Vue({})

// 定义好全局过滤器后, 组件内可以直接使用
<template>
  <div>
      {{num | fMoney}}
  </div>
</template>   

<script>
export default {
  data() {
    return {
      num: 1000
    }
  }, 
}
</script>

```

## (六) keep-alive(背诵) 还没讲

**问题**: 用户从列表的第3页, 点击某个商品进入了商品详情, 当用户点击返回的时候, 默认会返回到列表页的第一页而不是第3页, 这样的体验很不好, 所以我们希望可以回到列表页的原来位置, 这样的用户体验会比较好. 

**分析**: 之所以会回到第一页, 是因为返回到列表页的时候, 组件会重新创建, 从新执行created方法, 所以页面页重新渲染 

**解决**: 使用keep-alive可以缓存组件的状态, 具体做法:

​	 (1) 对列表页使用keep-alive, 使其即使离开了组件, 也不会销毁 组件挂载完毕的时候绑定滚动事件, 记录滚动的位置 

​	(2) 从详情页返回的时候, 滚动的原来的位置(在activated生命周期)

**注:** 

1. 被keep-live包裹的组件会被缓存
2. 使用keep-alive的组件crated和mounted只会执行一次
3. 离开组件会触发deactivated生命周期(只有被缓存的组件才有的生命周期)
4. 进入组件会触发activated生命周期

```
// 方法1 APP.vue
<template>
  <div id="app"> 
    <keep-alive>
      <router-view></router-view>
    </keep-alive>
  </div>
</template> 

// 方法2, 给路由配置keepAlive属性
// (1) /router/index.js
 {
    path: "/product",
    component: () => import("@/views/product/index.vue"),
    redirect: "/product/list",
    children: [
      {
        path: "list",
        // 缓存次组件
        meta: {
          keepAlive: true,
          tittle: '列表'
        },
        component: () => import("@/views/product/children/list2.vue"),
      },
      {
        path: "detail/:productId",
        component: () => import("@/views/product/children/detail.vue"),
      },
    ],
  },

// APP.vue
<template>
  <div id="app"> 
      <!-- 渲染需要缓存的组件 -->
     <keep-alive>  
        <router-view v-if="$route.meta.keepAlive"></router-view>
     </keep-alive>

      <!-- 渲染不需要缓存的组件 -->
      <router-view v-if="!$route.meta.keepAlive"></router-view>
  </div>
</template> 

```

```
// 上面需求的实现
(1) 在mounted绑定window.scroll事件, 滚动的时候保存滚动条的位置
(2) 返回时候, 重新滚动到原来保存的位置 

  mounted() {
    window.addEventListener('scroll',()=>{  
      // 保存滚动条位置
      if (window.scrollY>0) {
          this.scrollHeight = window.scrollY;
      }
    },false);
  },  

  // 进入组件
  activated() { 
    // 滚动到最初的位置
    setTimeout(()=> {
      window.scrollTo(0,this.scrollHeight); 
    },0)
  },
```

## (七) 配置环境变量

项目开发的时候, 一般会有多个环境, 比如开发环境, 测试环境, 生产环境, 我们调用接口的时候, 不同环境调用不同的接口, 所以要配置环境, ,方便访问。

**方式1:** 

```
// utils/http.js 核心代码
let env = process.env.NODE_ENV;
let baseURL;
// 开发环境
if (env === "development") {
  baseURL = "http://localhost:3003";
} else {
  baseURL = "http://huruqing.cn:3003";
}

const service = axios.create({
  // 如果换了新的项目, 需要更换为新的接口地址
  baseURL: baseURL,
  timeout: 50000, // 请求超时时间(因为需要调试后台,所以设置得比较大)
});
```

**方式2:**

(1) 在根目录添加env文件, 配置环境变量

// 开发环境 .env.development

```
VUE_APP_URL="http://192.168.2.21:3003"
```

// 生产环境 .env.production

```
# 配置生产地址,就是你将来发布上线的地址
NODE_ENV="production"
VUE_APP_URL="http://81.71.65.4:3003"
```

(2) 在项目的的任何一个地方, 都可以通过**process.env.VUE_APP_URL**来获取

// axios配置地址

```
const service = axios.create({
  // 配置基本路径
  baseURL: process.env.VUE_APP_URL, 
  timeout: 50000, // 请求超时时间(因为需要调试后台,所以设置得比较大)
});
```

 注意: 使用env文件配置环境变量, 缺点就是修改了地址, 需要重启项目, 所以我一般喜欢这样配置

process.env.NODE_ENV的值在开发中是"development", 在打包之后(生产), 它的值是"production", 下面的配置确定了生产环境的地址, 至于开发时的环境就任由我们填写了, 当然有时还会有"测试环境", 是开发好之后提供给测试人员去测试的环境

```
let env = process.env.NODE_ENV;
let baseURL;
if (env === 'production') {
  baseURL = "http://huruqing.cn:3003";
} else {
  baseURL = "http://192.168.2.21:3003";
}

const service = axios.create({ 
  baseURL,
  timeout: 50000,  
});
```



## (八) rem移动端适配

### (1) 元素单位有哪些:

px, vw和vh, 百分比, em和rem

### (2) rem和根标签字体大小的关系

```
// rem例子 demo1.html
<!DOCTYPE html>
<html lang="en" style="font-size: 100px;">
<head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <style> 
        div{
            width: 1rem;
            height: 1rem;
            background-color: gray;
        }
    </style>
</head>
<body>
    <div>

    </div>
</body>
</html>

// rem例子 demo1.html
<!DOCTYPE html>
<html lang="en" style="font-size: 112px;">
<head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <style> 
        div{
            width: 1rem;
            height: 1rem;
            background-color: green;
        }
    </style>
</head>
<body>
    <div>

    </div>
</body>
</html>

```

### (3) 移动端rem适配原理

1. 设置一个设备参考值(比如iPhone6), 设置它根字体大小
2. 写js代码获取设备的宽度, 跟据设备宽度等比缩放根标签字体大小

### (4) vue项目配置rem

1. 安装插 `npm i amfe-flexible --save`
2. 在main.js导入插件 `import 'amfe-flexible'`
3. px自动转rem
   1. 安装插件 `npm i postcss-pxtorem@5.1.1`
   2. 在/vue.config.js添加px2rem插件,把项目中的px转为rem

```
const pxtorem = require("postcss-pxtorem");
module.exports = {
  css: {
    loaderOptions: {
        // 后处理器配置
      postcss: {
        plugins: [
          // 把px转为rem
          pxtorem({
            rootValue: 37.5,
            propList: ["*"]
          })
        ]
      }
    }
  }
};
```

​	4.插件会修改html和body的字体大小, 而字体会继承, 所以要重新设置body的font-size为合适的字体大小

 

## (九) mixin(混入)

mixin 其实是一个对象，里面的结构大致跟普通组件的 script 里面的一样，有 data 属性，钩子函数和方法等 混入 (mixins) 是一种分发 Vue 组件中可复用功能的非常灵活的方式。混入对象可以包含任意组件选项。当组件使用混入对象时，所有混入对象的选项将被混入该组件本身的选项。

### 1.组件内混入

(1) 混入对象的生命周期先执行 

(2) data里的状态若有重名, 取的是组件里的状态

```
// mixin.js
export default {
    data: function() {
      return {
        username: "huruqing",
        age: 100
      };
    },

    created() {
        console.log('这是混入对象')
    },

    methods: {
        say() {
            console.log('hahahhahahha');
        }
    }
  };


// demo.vue
<template>
  <div>
    <p>{{username}}</p>
    <p>{{msg}}</p>
    <p>{{age}}</p>
  </div>
</template>

<script>
import mixin from './mixin'
export default {
  mixins:[mixin],
  data() {
    return {
      username: '张三',
      msg: 'hahahahahahha'
    }
  },

  created() {
    console.log('组件的created');
    this.say();
  }
}
</script> 
```

### 2.全局混入

```
// mixin.js
export default {
    methods: {
        loadingStart() {
            this.$toast.loading({
                message: '加载中...',
                forbidClick: true,
                duration:0
              });
        },
        loadingFinish() {
            this.$toast.clear();
        }
    }
}

// main.js,这个代码放在Vue.use(Vant)之后
import mixin from '@/utils/mixin';
Vue.mixin(mixin);

// 其他组件就可以直接使用下面代码来显示loading
this.loadingStart(); 
```

## (十) props检查类型

```
// demo.vue
<template>
  <div>
      <!-- 不传参数 -->
    <Son/> 
    <!-- 传了一个字符串 -->
    <!-- <Son :msg="'他是张三'" />  -->
    <!-- 传了一串数字 -->
    <!-- <Son :msg="22222"/> -->
  </div>
</template>

<script>
import Son from "./Son.vue";
export default {
  components: {
    Son,
  } 
};
</script> 

// Son.vue
<template>
  <div>{{msg}}</div>
</template>

<script>
export default {
    // props: ['msg']
    props:{
        msg: {
            type:String,
            default: 'hello'
        }
    }
}
</script> 
```

## (十一) nextStick

修改了数据之后, dom节点并不会立马更新, dom节点的更新是异步的, 想要拿到更新后的dom需要使用nextStick

```
<template>
  <div>
    <li ref="aa">{{ count1 }}</li>
    <li ref="bb">{{ count2 }}</li>
    <li ref="cc">{{ count3 }}</li>
    <button @click="handleClick">修改数据</button>
  </div>
</template>

<script>
export default {
  data() {
    return {
      count1: 0,
      count2: 0,
      count3: 0,
    };
  },

  methods: {
    handleClick() {
      this.count1 = 1;
      // this.count1=1;执行后dom并不会立即更新,dom节点的更新是异步的
      console.log(this.$refs.aa.innerHTML);  // 0
      // 当dom节点更新完毕, 会立即调用nextStick里的回调函数
      this.$nextTick(() => {
        console.log(this.$refs.aa.innerHTML);
      });
      this.count2 = 2;
      this.count3 = 3;
    },
  },
};
</script> 
```

## (十二) 配置跨域

 

http://81.71.65.4:3003/product/list

```
// vue.config.js里配置
devServer: {
    // 代理
    proxy: {
      // 只要请求地址有'api'都会匹配上
      "/api": {
        target: "http://81.71.65.4:3003",
        ws: true,
        // 允许跨域
        changeOrigin: true,
        pathRewrite: {
          "^/api": "", //通过pathRewrite重写地址，将前缀/api转为/
        },
      },
    }, 
  },
```

项目配置方法(一)

```
// api/index.js 的配置
let env = process.env.NODE_ENV; // development-开发环境, production-生产环境
let baseURL = "";
env = 'test';
if (env === "production") {
  // 生产接口
  baseURL = "http://huruqing.cn:3003";
} else if (env==='test'){
  // 本地跨域访问服务器端接口
  baseURL = '/api';
} else {
  // 开发环境接口(或者访问模拟数据接口)
  baseURL = "http://192.168.2.21:3003";
}

const service = axios.create({ 
  baseURL,
  timeout: 50000  
});
```

项目配置方法(二)

使用了env文件, 将.env.development文件修改即可

```
VUE_APP_URL="/api"
```



 注意: 请求接口时, 网络查看依然是本地的地址, 实际上请求的却是跨域里配置的地址, 比如请求以下接口

```
$axios.post('/api/product/list')
```

查看网络请求,  查看到的是 `http://localhost:8080/api/product/list`, 但实际却是通过代理请求的地址是

`http://81.71.65.4:3003/product/list`,  "api"被重写成了 "", 所以接口里就没有api这个字符串了



## (十三) 接口调试postMan

在极简插件里安装postMan

1. 添加环境变量
2. 新建接口
   - 添加参数
   - 添加请求头

![image-20221019160540445](http://192.168.2.21/chfs/shared/docs/img/image-20221019160540445.png)

 

 

